@@ -40,13 +40,12 @@ module GLib
     end
 
     def get_element_size
-      GLib.array_get_element_size self
+      Lib.g_array_get_element_size self
     end
 
     def self.wrap elmttype, ptr
-      super(ptr).tap do |it|
-        break if it.nil?
-        it.element_type = elmttype
+      super(ptr).tap do |array|
+        array.element_type = elmttype if array
       end
     end
 
