@@ -1,9 +1,8 @@
 module GLib
   module ContainerClassMethods
     def wrap typespec, ptr
-      super(ptr).tap do |it|
-        break if it.nil?
-        it.reset_typespec typespec
+      super(ptr).tap do |container|
+        container.reset_typespec typespec if container
       end
     end
 
@@ -14,6 +13,7 @@ module GLib
       when FFI::Pointer
         wrap typespec, it
       when self
+        # TODO: Reset or check?
         it.reset_typespec typespec
       else
         from_enumerable typespec, it
@@ -21,5 +21,3 @@ module GLib
     end
   end
 end
-
-
