@@ -1,3 +1,6 @@
+# NOTE: Do not require this file directly. Require 'gir_ffi' instead.
+#
+
 GirFFI.setup :GObject
 
 require 'ffi-gobject/base'
@@ -7,7 +10,6 @@ require 'ffi-gobject/initially_unowned'
 require 'ffi-gobject/closure'
 require 'ffi-gobject/object'
 require 'ffi-gobject/ruby_closure'
-require 'ffi-gobject/helper'
 require 'gir_ffi/builder/type/user_defined'
 
 module GObject
@@ -34,7 +36,7 @@ module GObject
   def self.type_from_instance_pointer inst_ptr
     return nil if inst_ptr.null?
     klsptr = inst_ptr.get_pointer 0
-    klsptr.send "get_#{GirFFI::TypeMap::TAG_TYPE_MAP[:gtype]}", 0
+    klsptr.send "get_#{GirFFI::TypeMap::TAG_TYPE_MAP[:GType]}", 0
   end
 
   def self.type_from_instance instance
@@ -51,31 +53,37 @@ module GObject
     ObjectClass.wrap klsptr
   end
 
-  _setup_method :signal_emitv
+  setup_method :signal_emitv
 
   def self.signal_lookup_from_instance signal, object
     signal_lookup signal, type_from_instance(object)
   end
 
-  def self.signal_emit object, signal, *args
-    id = signal_lookup_from_instance signal, object
-    arr = Helper.signal_arguments_to_gvalue_array signal, object, *args
-    rval = Helper.gvalue_for_signal_return_value signal, object
+  def self.signal_emit object, detailed_signal, *args
+    signal, detail = detailed_signal.split('::')
+    signal_id = signal_lookup_from_instance signal, object
+    detail_quark = GLib.quark_from_string(detail)
+    sig_info = object.class.find_signal signal
+    arr = sig_info.signal_arguments_to_gvalue_array object, *args
+    rval = sig_info.gvalue_for_signal_return_value
 
-    Lib.g_signal_emitv arr.values, id, 0, rval
+    Lib.g_signal_emitv arr.values, signal_id, detail_quark, rval
 
     return rval
   end
 
-  def self.signal_connect object, signal, data=nil, &block
-    callback = Helper.signal_callback object.class, signal, &block
+  def self.signal_connect object, detailed_signal, data=nil, &block
+    signal, _ = detailed_signal.split('::')
+    sig_info = object.class.find_signal signal
+    callback = sig_info.signal_callback(&block)
+    GirFFI::CallbackHelper.store_callback callback
+
     data_ptr = GirFFI::ArgHelper.object_to_inptr data
 
-    Lib::CALLBACKS << callback
-
-    Lib.g_signal_connect_data object, signal, callback, data_ptr, nil, 0
+    Lib.g_signal_connect_data object, detailed_signal, callback, data_ptr, nil, 0
   end
 
+  # Smells of :reek:LongParameterList: due to the C interface.
   def self.param_spec_int(name, nick, blurb, minimum, maximum,
                           default_value, flags)
     ptr = Lib.g_param_spec_int(name, nick, blurb, minimum, maximum,
@@ -90,17 +98,13 @@ module GObject
   load_class :ParamFlags
 
   module Lib
-    attach_function :g_type_init, [], :void
+    # NOTE: This Lib module is set up in `gir_ffi-base/gobject/lib.rb`.
+
     attach_function :g_object_ref, [:pointer], :void
     attach_function :g_object_ref_sink, [:pointer], :void
     attach_function :g_object_unref, [:pointer], :void
     attach_function :g_object_is_floating, [:pointer], :bool
 
-    # XXX: For older GObject/gobject-introspection.
-    unless method_defined? :g_object_newv
-      attach_function :g_object_newv, [:size_t, :uint, :pointer], :pointer
-    end
-
     attach_function :g_strv_get_type, [], :size_t
     attach_function :g_hash_table_get_type, [], :size_t
 
