@@ -8,13 +8,13 @@ module GObject
 
     include RubyStyle
 
-    _setup_method "new"
-    _setup_instance_method "get_property"
-    _setup_instance_method "set_property"
+    setup_method "new"
+    setup_instance_method "get_property"
+    setup_instance_method "set_property"
 
     def get_property_with_override property_name
       type = get_property_type property_name
-      gvalue = Helper.gvalue_for_type_info type
+      gvalue = type.make_g_value
 
       get_property_without_override property_name, gvalue
 
@@ -23,7 +23,7 @@ module GObject
 
     def set_property_with_override property_name, value
       type = get_property_type property_name
-      gvalue = Helper.gvalue_for_type_info type
+      gvalue = type.make_g_value
 
       gvalue.set_value adjust_value_to_type(value, type)
 
@@ -39,17 +39,16 @@ module GObject
     private
 
     def get_property_type property_name
-      prop = self.class._find_property property_name
+      prop = self.class.find_property property_name
       prop.property_type
     end
 
     def adjust_value_to_type val, type
       case type.tag
       when :ghash
-        GLib::HashTable.from [type.param_type(0).tag, type.param_type(1).tag],
-          val
+        GLib::HashTable.from type.element_type, val
       when :glist
-        GLib::List.from type.param_type(0).tag, val
+        GLib::List.from type.element_type, val
       else
         val
       end
