@@ -3,6 +3,7 @@ module GObject
 
   # Overrides for GValue, GObject's generic value container structure.
   class Value
+    # TODO: Give more generic name
     def set_ruby_value val
       if current_gtype == 0
         init_for_ruby_value val
@@ -17,15 +18,15 @@ module GObject
       TYPE_STRING => :set_string,
       TYPE_FLOAT => :set_float,
       TYPE_DOUBLE => :set_double,
-      TYPE_OBJECT => :set_instance,
+      TYPE_PARAM => :set_param,
+      TYPE_OBJECT => :set_instance_enhanced,
       TYPE_BOXED => :set_boxed,
       TYPE_POINTER => :set_pointer,
       TYPE_ENUM => :set_enum
     }
 
     def set_value val
-      method = TYPE_TO_SET_METHOD_MAP[current_fundamental_type]
-      call_or_raise method, val
+      send set_method, val
       self
     end
 
@@ -69,12 +70,13 @@ module GObject
       TYPE_POINTER => :get_pointer
     }
 
+    # TODO: Rename to get_value
     def ruby_value
-      method = TYPE_TO_GET_METHOD_MAP[current_fundamental_type]
-      call_or_raise method
+      send get_method
     end
 
     class << self
+      # TODO: Give more generic name
       def wrap_ruby_value val
         self.new.set_ruby_value val
       end
@@ -89,28 +91,56 @@ module GObject
           wrap_ruby_value val
         end
       end
+
+      def for_g_type g_type
+        return nil if g_type == TYPE_NONE
+        self.new.tap {|it| it.init g_type }
+      end
+    end
+
+    # TODO: Combine with wrap_ruby_value
+    def self.wrap_instance instance
+      self.new.tap {|it|
+        it.init GObject.type_from_instance instance
+        it.set_instance instance
+      }
     end
 
     private
 
+    def set_instance_enhanced val
+      check_type_compatibility val if val
+      set_instance val
+    end
+
+    def check_type_compatibility val
+      if !GObject::Value.type_compatible(GObject.type_from_instance(val), current_gtype)
+        raise ArgumentError, "#{val.class} is incompatible with #{current_gtype_name}"
+      end
+    end
+
     def get_boxed_enhanced
       boxed = get_boxed
-      case current_gtype
+      gtype = current_gtype
+
+      case gtype
       when TYPE_STRV
         GLib::Strv.wrap boxed
       when TYPE_HASH_TABLE
         GLib::HashTable.wrap [:gpointer, :gpointer], boxed
       else
-        GirFFI::ArgHelper.wrap_object_pointer_by_gtype boxed, current_gtype
+        GirFFI::ArgHelper.wrap_object_pointer_by_gtype boxed, gtype
       end
     end
 
-    def call_or_raise method, *args
-      if method
-        send method, *args
-      else
-        raise "Don't know how to handle #{current_gtype_name}"
-      end
+    def get_method
+      TYPE_TO_GET_METHOD_MAP[current_fundamental_type] or
+        raise "Can't find method to get #{current_gtype_name}"
+    end
+
+    def set_method
+      TYPE_TO_SET_METHOD_MAP[current_fundamental_type] or
+        raise "Can't find method to set #{current_gtype_name}"
     end
   end
 end
