@@ -2,8 +2,19 @@ module GObjectIntrospection
   # Wraps GIBaseInfo struct, the base \type for all info types.
   # Decendant types will be implemented as needed.
   class IBaseInfo
-    def initialize ptr
+    def initialize ptr, lib=Lib
+      raise ArgumentError, "ptr must not be null" if ptr.null?
+
+      unless defined?(RUBY_ENGINE) && RUBY_ENGINE == 'rbx'
+        ObjectSpace.define_finalizer self, self.class.make_finalizer(lib, ptr)
+      end
+
       @gobj = ptr
+      @lib = lib
+    end
+
+    def self.make_finalizer lib, ptr
+      proc { lib.g_base_info_unref ptr }
     end
 
     def to_ptr
@@ -23,28 +34,18 @@ module GObjectIntrospection
       single ||= method[0..-2]
       count = method.sub(/^(get_)?/, "\\1n_")
       self.class_eval <<-CODE
-	def #{method}
-	  (0..(#{count} - 1)).map do |i|
-	    #{single} i
-	  end
-	end
+        def #{method}
+          (0..(#{count} - 1)).map do |i|
+            #{single} i
+          end
+        end
       CODE
     end
 
-    private_class_method :new
-
     def name
       Lib.g_base_info_get_name @gobj
     end
 
-    def safe_name
-      if name =~ /^_/
-        "Private__" + name
-      else
-        name.gsub(/^(.)/) { $1.upcase }
-      end
-    end
-
     def info_type
       Lib.g_base_info_get_type @gobj
     end
