@@ -3,19 +3,7 @@ require 'ffi'
 module GObjectIntrospection
   module Lib
     extend FFI::Library
-    begin
-      ffi_lib "girepository-1.0.so.1"
-    rescue LoadError
-      begin
-        ffi_lib "girepository-1.0.so.0"
-        warn "This old version of gobject-introspection is not supported by GirFFI."
-        warn "Please upgrade to at least version 0.10.0."
-      rescue LoadError
-        ffi_lib "girepository-1.0"
-        warn "This platform and/or version of gobject-introspection are not supported by GirFFI."
-        warn "Please file bugs for any errors found."
-      end
-    end
+    ffi_lib "girepository-1.0"
 
     # IRepository
     enum :IRepositoryLoadFlags, [:LAZY, (1<<0)]
@@ -61,6 +49,7 @@ module GObjectIntrospection
       :unresolved
     ]
 
+    attach_function :g_base_info_unref, [:pointer], :void
     attach_function :g_base_info_get_type, [:pointer], :IInfoType
     attach_function :g_base_info_get_name, [:pointer], :string
     attach_function :g_base_info_get_namespace, [:pointer], :string
@@ -116,8 +105,13 @@ module GObjectIntrospection
     # retrieve the correct values before declaring the ITypeTag enum.
     attach_function :tmp_type_tag_to_string, :g_type_tag_to_string, [:int], :string
     type_tag_map = (0..31).map { |id|
-      [tmp_type_tag_to_string(id).to_sym, id]
-    }.flatten
+      sym = tmp_type_tag_to_string(id).to_sym
+      if sym == :unknown
+        nil
+      else
+        [sym, id]
+      end
+    }.compact.flatten
     enum :ITypeTag, type_tag_map
 
     # Now, attach g_type_tag_to_string again under its own name with an
@@ -174,7 +168,7 @@ module GObjectIntrospection
     attach_function :g_union_info_find_method, [:pointer, :string], :pointer
     attach_function :g_union_info_get_size, [:pointer], :int
     attach_function :g_union_info_get_alignment, [:pointer], :int
- 
+
     # IRegisteredTypeInfo
     attach_function :g_registered_type_info_get_type_name, [:pointer], :string
     attach_function :g_registered_type_info_get_type_init, [:pointer], :string
@@ -239,6 +233,7 @@ module GObjectIntrospection
     attach_function :g_interface_info_get_constant, [:pointer, :int], :pointer
     attach_function :g_interface_info_get_iface_struct, [:pointer], :pointer
 
+    # Union type representing an argument value
     class GIArgument < FFI::Union
       signed_size_t = "int#{FFI.type_size(:size_t) * 8}".to_sym
 
