@@ -11,12 +11,14 @@ module GirFFI
     SIMPLE_G_TYPES = [
       :gint8, :gint16, :gint, :gint32, :gint64,
       :guint8, :guint16, :guint32, :guint64,
-      :gfloat, :gdouble]
+      :gfloat, :gdouble
+    ]
 
-    # FIXME: Hideous.
+    # FIXME: Hideous
+    # TODO: Move this implementation to InPointer
     def self.object_to_inptr obj
-      return obj.to_ptr if obj.respond_to? :to_ptr
       return nil if obj.nil?
+      return obj.to_ptr if obj.respond_to? :to_ptr
       return obj if obj.is_a? FFI::Pointer
 
       FFI::Pointer.new(obj.object_id).tap {|ptr|
@@ -24,11 +26,14 @@ module GirFFI
     end
 
     def self.ptr_to_typed_array type, ptr, size
-      if type.is_a? Class
+      case type
+      when Class
         ptr_to_interface_array type, ptr, size
-      elsif type.is_a? Array
+      when Array
         ptr_to_interface_pointer_array type[1], ptr, size
-      elsif type == :utf8
+      when FFI::Enum
+        ptr_to_enum_array type, ptr, size
+      when :utf8
         ptr_to_utf8_array ptr, size
       else
         self.send "ptr_to_#{type}_array", ptr, size
@@ -57,7 +62,7 @@ module GirFFI
 
     def self.ptr_to_interface_array klass, ptr, size
       return [] if ptr.nil? or ptr.null?
-      struct_size = klass.ffi_structure.size
+      struct_size = klass::Struct.size
       size.times.map do |idx|
         klass.wrap(ptr + struct_size * idx)
       end
@@ -71,6 +76,10 @@ module GirFFI
       end
     end
 
+    def self.ptr_to_enum_array enum, ptr, size
+      ptr_to_gint32_array(ptr, size).map {|val| enum[val] }
+    end
+
     if RUBY_VERSION < "1.9"
       def self.ptr_to_utf8 ptr
         ptr.null? ? nil : ptr.read_string
@@ -94,12 +103,12 @@ module GirFFI
 
     def self.check_error errpp
       err = GLib::Error.wrap(errpp.read_pointer)
-      raise err.message unless err.nil?
+      raise err.message if err
     end
 
     def self.check_fixed_array_size size, arr, name
       unless arr.size == size
-	raise ArgumentError, "#{name} should have size #{size}"
+        raise ArgumentError, "#{name} should have size #{size}"
       end
     end
 
@@ -115,7 +124,7 @@ module GirFFI
     def self.wrap_object_pointer_by_gtype optr, gtype
       return nil if optr.null?
       klass = Builder.build_by_gtype gtype
-      klass.wrap optr
+      klass.direct_wrap optr
     end
 
     def self.cast_from_pointer type, it
