@@ -1,18 +1,19 @@
-require 'gir_ffi/builder_helper'
-require 'gir_ffi/builder/type/base'
 require 'gir_ffi/builder/type/callback'
 require 'gir_ffi/builder/type/constant'
 require 'gir_ffi/builder/type/enum'
-require 'gir_ffi/builder/type/union'
+require 'gir_ffi/builder/type/interface'
 require 'gir_ffi/builder/type/object'
 require 'gir_ffi/builder/type/struct'
-require 'gir_ffi/builder/type/interface'
+require 'gir_ffi/builder/type/unintrospectable'
+require 'gir_ffi/builder/type/union'
 
 module GirFFI
   # Builds a class based on information found in the introspection
   # repository.
   module Builder
     module Type
+      CACHE = {}
+
       TYPE_MAP = {
         :callback => Callback,
         :constant => Constant,
@@ -21,7 +22,8 @@ module GirFFI
         :interface => Interface,
         :object => Object,
         :struct => Struct,
-        :union => Union
+        :union => Union,
+        :unintrospectable => Unintrospectable
       }
 
       def self.build info
