@@ -13,24 +13,16 @@ module GirFFI
           end
         end
 
-        def pretty_print
-          args = argument_types.map do |arg|
-            arg.is_a?(FFI::Enum) ? arg.tag : arg.inspect
-          end
-          return "#{@classname} = Lib.callback #{callback_sym.inspect}, " +
-            "[#{args.join(', ')}], #{return_type.inspect}"
-        end
-
         def callback_sym
           @classname.to_sym
         end
 
         def argument_types
-          Builder.ffi_function_argument_types info
+          @info.argument_ffi_types
         end
 
         def return_type
-          Builder.ffi_function_return_type info
+          @info.return_ffi_type
         end
       end
     end
