@@ -1,4 +1,5 @@
 require 'gir_ffi/builder/type/registered_type'
+
 module GirFFI
   module Builder
     module Type
@@ -7,10 +8,6 @@ module GirFFI
       # attached to the appropriate namespace module, and will be defined
       # as an enum for FFI.
       class Enum < RegisteredType
-        def pretty_print
-          "#{@classname} = Lib.enum #{enum_sym.inspect}, #{value_spec.inspect}"
-        end
-
         private
 
         def enum_sym
@@ -28,7 +25,18 @@ module GirFFI
           @klass = optionally_define_constant namespace_module, @classname do
             lib.enum(enum_sym, value_spec)
           end
-          setup_gtype_getter unless already_set_up
+          unless already_set_up
+            setup_gtype_getter
+            setup_inspect
+          end
+        end
+
+        def setup_inspect
+          @klass.instance_eval <<-EOS
+            def self.inspect
+              "#{@namespace}::#{@classname}"
+            end
+          EOS
         end
 
         def already_set_up
