@@ -41,7 +41,7 @@ module GirFFI
           return if info.find_method 'new'
 
           (class << @klass; self; end).class_eval {
-            alias_method :new, :allocate
+            alias_method :new, :_allocate
           }
         end
 
@@ -54,19 +54,9 @@ module GirFFI
         end
 
         def superclass
-          unless defined? @superclass
-            if parent
-              @superclass = Builder.build_class parent
-            else
-              @superclass = GirFFI::ClassBase
-            end
-          end
-          @superclass
+          @superclass ||= ClassBase
         end
       end
     end
   end
 end
-
-
-
