@@ -9,18 +9,10 @@ module GirFFI
     extend Forwardable
     def_delegators :@struct, :to_ptr
 
-    def ffi_structure
-      self.class.ffi_structure
-    end
-
-    def _builder
-      self.class._builder
-    end
-
     def setup_and_call method, *arguments, &block
       result = self.class.ancestors.any? do |klass|
-        klass.respond_to?(:_setup_instance_method) &&
-          klass._setup_instance_method(method.to_s)
+        klass.respond_to?(:setup_instance_method) &&
+          klass.setup_instance_method(method.to_s)
       end
 
       unless result
@@ -32,8 +24,8 @@ module GirFFI
 
     def self.setup_and_call method, *arguments, &block
       result = self.ancestors.any? do |klass|
-        klass.respond_to?(:_setup_method) &&
-          klass._setup_method(method.to_s)
+        klass.respond_to?(:setup_method) &&
+          klass.setup_method(method.to_s)
       end
 
       unless result
@@ -44,52 +36,43 @@ module GirFFI
     end
 
     class << self
-      def ffi_structure
-	self.const_get(:Struct)
-      end
-
       def gir_info
-	self.const_get :GIR_INFO
+        self.const_get :GIR_INFO
       end
 
-      def _builder
-	self.const_get :GIR_FFI_BUILDER
+      def gir_ffi_builder
+        self.const_get :GIR_FFI_BUILDER
       end
 
-      def _find_signal name
-        _builder.find_signal name
+      def setup_method name
+        gir_ffi_builder.setup_method name
       end
 
-      def _find_property name
-        _builder.find_property name
-      end
-
-      def _setup_method name
-        _builder.setup_method name
-      end
-
-      def _setup_instance_method name
-        _builder.setup_instance_method name
+      def setup_instance_method name
+        gir_ffi_builder.setup_instance_method name
       end
 
       alias_method :_real_new, :new
       undef new
 
+      # Wrap the passed pointer in an instance of the current class, or a
+      # descendant type if applicable.
       def wrap ptr
-	return nil if ptr.nil? or ptr.null?
-	obj = _real_new
-        obj.instance_variable_set :@struct, ffi_structure.new(ptr.to_ptr)
+        direct_wrap ptr
+      end
+
+      # Wrap the passed pointer in an instance of the current class. Will not
+      # do any casting to subtypes.
+      def direct_wrap ptr
+        return nil if !ptr or ptr.null?
+        obj = _real_new
+        obj.instance_variable_set :@struct, self::Struct.new(ptr.to_ptr)
         obj
       end
 
-      # TODO: Only makes sense for :objects.
-      def constructor_wrap ptr
-        wrap ptr
-      end
-
-      def allocate
-	obj = _real_new
-        obj.instance_variable_set :@struct, ffi_structure.new
+      def _allocate
+        obj = _real_new
+        obj.instance_variable_set :@struct, self::Struct.new
         obj
       end
 
