@@ -1,29 +1,32 @@
 # coding: utf-8
-require File.expand_path('../gir_ffi_test_helper.rb', File.dirname(__FILE__))
+require 'gir_ffi_test_helper'
 
 require 'gir_ffi'
 
 GirFFI.setup :GIMarshallingTests
 
 # Tests generated methods and functions in the GIMarshallingTests namespace.
-describe "GIMarshallingTests" do
-  describe "BoxedStruct" do
-    it "is created with #new" do
+describe GIMarshallingTests do
+  describe "GIMarshallingTests::BoxedStruct" do
+    it "creates an instance using #new" do
       bx = GIMarshallingTests::BoxedStruct.new
       assert_instance_of GIMarshallingTests::BoxedStruct, bx
     end
 
+    let(:instance) { GIMarshallingTests::BoxedStruct.new }
+
+    it "has a working method #inv" do
+      instance.long_ = 42
+      instance.inv
+      pass
+    end
+
     describe "an instance" do
       before do
-        @bx = GIMarshallingTests::BoxedStruct.new
+        @bx = instance
         @bx.long_ = 42
       end
 
-      it "has a working method #inv" do
-        @bx.inv
-        pass
-      end
-
       it "has a writable field long_" do
         assert_equal 42, @bx.long_
         @bx.long_ = 43
@@ -38,7 +41,6 @@ describe "GIMarshallingTests" do
     end
 
     it "has a working function #inout" do
-      skip unless get_method_introspection_data 'GIMarshallingTests', 'BoxedStruct', 'inout'
       bx = GIMarshallingTests::BoxedStruct.new
       bx.long_ = 42
 
@@ -49,13 +51,11 @@ describe "GIMarshallingTests" do
     end
 
     it "has a working function #out" do
-      skip unless get_method_introspection_data 'GIMarshallingTests', 'BoxedStruct', 'out'
       res = GIMarshallingTests::BoxedStruct.out
       assert_equal 42, res.long_
     end
 
     it "has a working function #returnv" do
-      skip unless get_method_introspection_data 'GIMarshallingTests', 'BoxedStruct', 'returnv'
       res = GIMarshallingTests::BoxedStruct.returnv
       assert_equal 42, res.long_
       assert_equal ["0", "1", "2"], res.g_strv.to_a
@@ -66,6 +66,11 @@ describe "GIMarshallingTests" do
     assert_equal 5, GIMarshallingTests::CONSTANT_GERROR_CODE
   end
 
+  it "has the constant CONSTANT_GERROR_DEBUG_MESSAGE" do
+    GIMarshallingTests::CONSTANT_GERROR_DEBUG_MESSAGE.must_equal(
+      "we got an error, life is shit")
+  end
+
   it "has the constant CONSTANT_GERROR_DOMAIN" do
     assert_equal "gi-marshalling-tests-gerror-domain",
       GIMarshallingTests::CONSTANT_GERROR_DOMAIN
@@ -84,35 +89,101 @@ describe "GIMarshallingTests" do
     assert_equal "const ♥ utf8", GIMarshallingTests::CONSTANT_UTF8
   end
 
-  it "has the enum Enum" do
-    assert_equal 0, GIMarshallingTests::Enum[:value1]
-    assert_equal 1, GIMarshallingTests::Enum[:value2]
-    assert_equal 42, GIMarshallingTests::Enum[:value3]
+  describe "GIMarshallingTests::Enum" do
+    it "has the member :value1" do
+      assert_equal 0, GIMarshallingTests::Enum[:value1]
+    end
+
+    it "has the member :value2" do
+      assert_equal 1, GIMarshallingTests::Enum[:value2]
+    end
+
+    it "has the member :value3" do
+      assert_equal 42, GIMarshallingTests::Enum[:value3]
+    end
   end
 
-  it "has the bitfield Flags" do
-    assert_equal 1, GIMarshallingTests::Flags[:value1]
-    assert_equal 2, GIMarshallingTests::Flags[:value2]
-    assert_equal 4, GIMarshallingTests::Flags[:value3]
-    assert_equal 3, GIMarshallingTests::Flags[:mask]
-    assert_equal 3, GIMarshallingTests::Flags[:mask2]
+  describe "GIMarshallingTests::Flags" do
+    it "has the member :value1" do
+      assert_equal 1, GIMarshallingTests::Flags[:value1]
+    end
+
+    it "has the member :value2" do
+      assert_equal 2, GIMarshallingTests::Flags[:value2]
+    end
+
+    it "has the member :value3" do
+      assert_equal 4, GIMarshallingTests::Flags[:value3]
+    end
+
+    it "has the member :mask" do
+      assert_equal 3, GIMarshallingTests::Flags[:mask]
+    end
+
+    it "has the member :mask2" do
+      assert_equal 3, GIMarshallingTests::Flags[:mask2]
+    end
+
+    it "has a working function #in" do
+      skip "methods on enums are not supported yet"
+    end
+    it "has a working function #in_zero" do
+      skip "methods on enums are not supported yet"
+    end
+    it "has a working function #inout" do
+      skip "methods on enums are not supported yet"
+    end
+    it "has a working function #out" do
+      skip "methods on enums are not supported yet"
+    end
+    it "has a working function #returnv" do
+      skip "methods on enums are not supported yet"
+    end
   end
 
-  it "has the enum GEnum" do
-    assert_equal 0, GIMarshallingTests::GEnum[:value1]
-    assert_equal 1, GIMarshallingTests::GEnum[:value2]
-    assert_equal 42, GIMarshallingTests::GEnum[:value3]
+  describe "GIMarshallingTests::GEnum" do
+    it "has the member :value1" do
+      assert_equal 0, GIMarshallingTests::GEnum[:value1]
+    end
+    it "has the member :value2" do
+      assert_equal 1, GIMarshallingTests::GEnum[:value2]
+    end
+    it "has the member :value3" do
+      assert_equal 42, GIMarshallingTests::GEnum[:value3]
+    end
+    it "has a working function #in" do
+      skip "methods on enums are not supported yet"
+    end
+    it "has a working function #inout" do
+      skip "methods on enums are not supported yet"
+    end
+    it "has a working function #out" do
+      skip "methods on enums are not supported yet"
+    end
+    it "has a working function #returnv" do
+      skip "methods on enums are not supported yet"
+    end
   end
 
-  describe "Interface" do
-    it "must be tested"
+  describe "GIMarshallingTests::Interface" do
+    it "has a working method #test_int8_in" do
+      skip
+    end
   end
 
-  describe "Interface2" do
-    it "must be tested"
+  describe "GIMarshallingTests::Interface2" do
+    it "must be tested" do
+      skip
+    end
   end
 
-  describe "NestedStruct" do
+  describe "GIMarshallingTests::Interface3" do
+    it "has a working method #test_variant_array_in" do
+      skip
+    end
+  end
+
+  describe "GIMarshallingTests::NestedStruct" do
     it "contains a SimpleStruct" do
       ns = GIMarshallingTests::NestedStruct.new
       assert_instance_of GIMarshallingTests::SimpleStruct,
@@ -120,15 +191,25 @@ describe "GIMarshallingTests" do
     end
   end
 
-  it "has the bitfield NoTypeFlags" do
-    assert_equal 1, GIMarshallingTests::NoTypeFlags[:value1]
-    assert_equal 2, GIMarshallingTests::NoTypeFlags[:value2]
-    assert_equal 4, GIMarshallingTests::NoTypeFlags[:value3]
-    assert_equal 3, GIMarshallingTests::NoTypeFlags[:mask]
-    assert_equal 3, GIMarshallingTests::NoTypeFlags[:mask2]
+  describe "GIMarshallingTests::NoTypeFlags" do
+    it "has the member :value1" do
+      assert_equal 1, GIMarshallingTests::NoTypeFlags[:value1]
+    end
+    it "has the member :value2" do
+      assert_equal 2, GIMarshallingTests::NoTypeFlags[:value2]
+    end
+    it "has the member :value3" do
+      assert_equal 4, GIMarshallingTests::NoTypeFlags[:value3]
+    end
+    it "has the member :mask" do
+      assert_equal 3, GIMarshallingTests::NoTypeFlags[:mask]
+    end
+    it "has the member :mask2" do
+      assert_equal 3, GIMarshallingTests::NoTypeFlags[:mask2]
+    end
   end
 
-  describe "NotSimpleStruct" do
+  describe "GIMarshallingTests::NotSimpleStruct" do
     it "must be tested"
   end
 
@@ -136,8 +217,8 @@ describe "GIMarshallingTests" do
     assert_equal 42, GIMarshallingTests::OVERRIDES_CONSTANT
   end
 
-  describe "Object" do
-    it "creates instances with #new" do
+  describe "GIMarshallingTests::Object" do
+    it "creates an instance using #new" do
       ob = GIMarshallingTests::Object.new 42
       assert_instance_of GIMarshallingTests::Object, ob
       assert_equal 42, ob.int
@@ -160,8 +241,9 @@ describe "GIMarshallingTests" do
       assert_instance_of GIMarshallingTests::Object, res
     end
 
-    # This function is only found in the header
-    # it "has a working function #inout_same"
+    it "has a working function #inout_same" do
+      skip "This function is only found in the header"
+    end
 
     it "has a working function #none_inout" do
       ob = GIMarshallingTests::Object.new 42
@@ -185,279 +267,349 @@ describe "GIMarshallingTests" do
       pass
     end
 
-    it "has a working function #static_method" do
-      GIMarshallingTests::Object.static_method
+    let(:instance) { GIMarshallingTests::Object.new 42 }
+
+    it "has a working method #call_vfunc_with_callback" do
+      # NOTE: To call this method, the callback slot vfunc_with_callback has to
+      # be filled in the GIMarshallingTests::Object class structure. The
+      # GIMarshallingTests library doesn't do this.
+      skip "Needs vfunc setup"
+    end
+
+    it "has a working method #full_in" do
+      skip "This function is only found in the header"
+    end
+
+    it "has a working method #int8_in" do
+      skip "Needs vfunc setup"
+    end
+
+    it "has a working method #int8_out" do
+      skip "Needs vfunc setup"
+    end
+
+    # TODO: Avoid using common method names?
+    it "has a working method #method" do
+      instance.method
       pass
     end
 
-    describe "an instance" do
-      before do
-        @obj = GIMarshallingTests::Object.new 42
-      end
+    it "has a working method #method_array_in" do
+      instance.method_array_in [-1, 0, 1, 2]
+      pass
+    end
 
-      it "has a working virtual method #method_int8_in"
-      it "has a working virtual method #method_int8_out"
+    it "has a working method #method_array_inout" do
+      res = instance.method_array_inout [-1, 0, 1, 2]
+      assert_equal [-2, -1, 0, 1, 2], res.to_a
+    end
 
-      it "has a working virtual method #method_with_default_implementation" do
-        @obj.method_with_default_implementation 104
-        assert_equal 104, @obj.int
-      end
+    it "has a working method #method_array_out" do
+      res = instance.method_array_out
+      assert_equal [-1, 0, 1, 2], res.to_a
+    end
 
-      # This function is only found in the header
-      # it "has a working method #full_in"
+    it "has a working method #method_array_return" do
+      res = instance.method_array_return
+      assert_equal [-1, 0, 1, 2], res.to_a
+    end
 
-      it "has a working method #int8_in"
-      it "has a working method #int8_out"
+    it "has a working method #method_int8_in" do
+      skip "Needs vfunc setup"
+    end
 
-      # TODO: Avoid using common method names?
-      it "has a working method #method" do
-        @obj.method
-        pass
-      end
+    it "has a working method #method_int8_out" do
+      skip "Needs vfunc setup"
+    end
 
-      it "has a working method #method_array_in" do
-        @obj.method_array_in [-1, 0, 1, 2]
-        pass
-      end
+    it "has a working method #method_variant_array_in" do
+      skip "Needs vfunc setup"
+    end
 
-      it "has a working method #method_array_inout" do
-        res = @obj.method_array_inout [-1, 0, 1, 2]
-        assert_equal [-2, -1, 0, 1, 2], res
-      end
+    it "has a working method #method_with_default_implementation" do
+      instance.method_with_default_implementation 104
+      assert_equal 104, instance.int
+    end
 
-      it "has a working method #method_array_out" do
-        res = @obj.method_array_out
-        assert_equal [-1, 0, 1, 2], res
-      end
+    it "has a working method #none_in" do
+      instance.none_in
+      pass
+    end
 
-      it "has a working method #method_array_return" do
-        res = @obj.method_array_return
-        assert_equal [-1, 0, 1, 2], res
-      end
+    it "has a working method #overridden_method" do
+      instance.set_property("int", 0)
+      instance.overridden_method
+      pass
+    end
 
-      it "has a working method #none_in" do
-        @obj.none_in
-        pass
-      end
+    it "has a working method #vfunc_caller_allocated_out_parameter" do
+      skip "Needs vfunc setup"
+    end
+    it "has a working method #vfunc_meth_with_error" do
+      skip "Needs vfunc setup"
+    end
+    it "has a working method #vfunc_multiple_out_parameters" do
+      skip "Needs vfunc setup"
+    end
+    it "has a working method #vfunc_one_out_parameter" do
+      skip "Needs vfunc setup"
+    end
+    it "has a working method #vfunc_return_value_and_multiple_out_parameters" do
+      skip "Needs vfunc setup"
+    end
+    it "has a working method #vfunc_return_value_and_one_out_parameter" do
+      skip "Needs vfunc setup"
+    end
+    it "has a working method #vfunc_return_value_only" do
+      skip "Needs vfunc setup"
+    end
+    it "has a working method #vfunc_with_callback" do
+      skip "Needs vfunc setup"
+    end
 
-      it "has a property 'int' containing the argument to #new" do
-        assert_equal 42, @obj.get_property("int")
-        @obj.set_property("int", 13)
-        assert_equal 13, @obj.get_property("int")
-      end
+    it "has a property 'int' containing the argument to #new" do
+      assert_equal 42, instance.get_property("int")
+      instance.set_property("int", 13)
+      assert_equal 13, instance.get_property("int")
+    end
 
-      it "has a working method #overridden_method" do
-        @obj.set_property("int", 0)
-        @obj.overridden_method
-        pass
-      end
-
-      it "has property accessors for the int property" do
-        assert_equal 42, @obj.int
-        @obj.int = 1
-        assert_equal 1, @obj.int
-      end
+    it "has property accessors for the int property" do
+      assert_equal 42, instance.int
+      instance.int = 1
+      assert_equal 1, instance.int
     end
   end
 
-  describe "OverridesObject" do
-    it "creates instances with #new" do
+  describe "GIMarshallingTests::OverridesObject" do
+    it "creates an instance using #new" do
       ob = GIMarshallingTests::OverridesObject.new
       assert_instance_of GIMarshallingTests::OverridesObject, ob
     end
 
-    it "creates instances with #returnv" do
+    it "has a working function #returnv" do
       ob = GIMarshallingTests::OverridesObject.returnv
       assert_instance_of GIMarshallingTests::OverridesObject, ob
     end
 
-    describe "an instance" do
-      before do
-        @obj = GIMarshallingTests::OverridesObject.new
-      end
+    let(:instance) { GIMarshallingTests::OverridesObject.new }
 
-      it "does not have field accessor methods" do
-        assert_raises(NoMethodError) { @obj.parent_instance }
-        assert_raises(NoMethodError) { @obj.parent_instance = nil }
-        assert_raises(NoMethodError) { @obj.long_ }
-        assert_raises(NoMethodError) { @obj.long_ = 1 }
-      end
+    it "has a working method #method" do
+      result = instance.method
+      result.must_equal 42
+    end
+
+    it "does not have field accessor methods" do
+      assert_raises(NoMethodError) { instance.parent_instance }
+      assert_raises(NoMethodError) { instance.parent_instance = nil }
+      assert_raises(NoMethodError) { instance.long_ }
+      assert_raises(NoMethodError) { instance.long_ = 1 }
     end
   end
 
-  describe "PointerStruct" do
-    it "creates instances with #new" do
+  describe "GIMarshallingTests::OverridesStruct" do
+    let(:instance) { GIMarshallingTests::OverridesStruct.new }
+
+    it "creates an instance using #new" do
+      ob = GIMarshallingTests::OverridesStruct.new
+      assert_instance_of GIMarshallingTests::OverridesStruct, ob
+    end
+
+    it "has a working method #method" do
+      instance.method.must_equal 42
+    end
+
+    it "has a working function #returnv" do
+      ob = GIMarshallingTests::OverridesStruct.returnv
+      assert_instance_of GIMarshallingTests::OverridesStruct, ob
+    end
+
+    it "has a writable field long_" do
+      instance.long_ = 43
+      instance.long_.must_equal 43
+    end
+  end
+
+  describe "GIMarshallingTests::PointerStruct" do
+    it "creates an instance using #new" do
       ps = GIMarshallingTests::PointerStruct.new
       assert_instance_of GIMarshallingTests::PointerStruct, ps
     end
 
-    describe "an instance" do
-      before do
-        @ps = GIMarshallingTests::PointerStruct.new
-      end
+    let(:instance) { GIMarshallingTests::PointerStruct.new }
 
-      it "has a writable field long_" do
-        assert_equal 0, @ps.long_
-        @ps.long_ = 1056
-        assert_equal 1056, @ps.long_
-      end
+    it "has a working method #inv" do
+      instance.long_ = 42
+      instance.inv
+      pass
+    end
 
-      it "has a working method #inv" do
-        @ps.long_ = 42
-        @ps.inv
-        pass
-      end
+    it "has a writable field long_" do
+      assert_equal 0, instance.long_
+      instance.long_ = 1056
+      assert_equal 1056, instance.long_
+    end
+
+    it "has a working function #returnv" do
+      ob = GIMarshallingTests::PointerStruct.returnv
+      assert_instance_of GIMarshallingTests::PointerStruct, ob
     end
   end
 
-  it "has the enum SecondEnum" do
-    assert_equal 0, GIMarshallingTests::SecondEnum[:secondvalue1]
-    assert_equal 1, GIMarshallingTests::SecondEnum[:secondvalue2]
+  describe "GIMarshallingTests::PropertiesObject" do
+    it "creates an instance using #new" do
+      ob = GIMarshallingTests::PropertiesObject.new
+      assert_instance_of GIMarshallingTests::PropertiesObject, ob
+    end
+
+    it "has the property 'some-boolean'"
+    it "has the property 'some-boxed-struct'"
+    it "has the property 'some-char'"
+    it "has the property 'some-double'"
+    it "has the property 'some-float'"
+    it "has the property 'some-int'"
+    it "has the property 'some-int64'"
+    it "has the property 'some-long'"
+    it "has the property 'some-strv'"
+    it "has the property 'some-uchar'"
+    it "has the property 'some-uint'"
+    it "has the property 'some-uint64'"
+    it "has the property 'some-ulong'"
   end
 
-  describe "SimpleStruct" do
-    it "creates instances with #new" do
+  describe "GIMarshallingTests::SecondEnum" do
+    it "has the member :secondvalue1" do
+      assert_equal 0, GIMarshallingTests::SecondEnum[:secondvalue1]
+    end
+    it "has the member :secondvalue2" do
+      assert_equal 1, GIMarshallingTests::SecondEnum[:secondvalue2]
+    end
+  end
+
+  describe "GIMarshallingTests::SimpleStruct" do
+    it "creates an instance using #new" do
       ss = GIMarshallingTests::SimpleStruct.new
       assert_instance_of GIMarshallingTests::SimpleStruct, ss
     end
 
-    describe "an instance" do
-      before do
-        @ss = GIMarshallingTests::SimpleStruct.new
-      end
+    let(:instance) { GIMarshallingTests::SimpleStruct.new }
 
-      it "has a writable field long_" do
-        assert_equal 0, @ss.long_
-        @ss.long_ = 1056
-        assert_equal 1056, @ss.long_
-      end
+    it "has a working method #inv" do
+      instance.long_ = 6
+      instance.int8 = 7
+      instance.inv
+      pass
+    end
 
-      it "has a writable field int8" do
-        assert_equal 0, @ss.int8
-        @ss.int8 = -43
-        assert_equal(-43, @ss.int8)
-      end
+    it "has a working method #method" do
+      instance.long_ = 6
+      instance.int8 = 7
+      instance.method
+      pass
+    end
 
-      it "has a working method #inv" do
-        @ss.long_ = 6
-        @ss.int8 = 7
-        @ss.inv
-        pass
-      end
+    it "has a writable field long_" do
+      assert_equal 0, instance.long_
+      instance.long_ = 1056
+      assert_equal 1056, instance.long_
+    end
+
+    it "has a writable field int8" do
+      assert_equal 0, instance.int8
+      instance.int8 = -43
+      assert_equal(-43, instance.int8)
+    end
+
+    it "has a working function #returnv" do
+      ss = GIMarshallingTests::SimpleStruct.returnv
+      assert_instance_of GIMarshallingTests::SimpleStruct, ss
     end
   end
 
-  describe "SubObject" do
-    it "creates instances with #new" do
+  describe "GIMarshallingTests::SubObject" do
+    it "creates an instance using #new" do
       so = GIMarshallingTests::SubObject.new 42
       assert_instance_of GIMarshallingTests::SubObject, so
     end
 
-    describe "an instance" do
-      before do
-        @so = GIMarshallingTests::SubObject.new 0
-      end
+    let(:instance) { GIMarshallingTests::SubObject.new 0 }
 
-      it "has the method #overwritten_method" do
-        @so.overwritten_method
-        pass
-      end
+    it "has a working method #overwritten_method" do
+      instance.overwritten_method
+      pass
+    end
 
-      it "has the method #sub_method" do
-        @so.sub_method
-        pass
-      end
+    it "has a working method #sub_method" do
+      instance.sub_method
+      pass
+    end
 
-      it "does not have field accessors" do
-        assert_raises(NoMethodError) { @so.parent_instance }
-        assert_raises(NoMethodError) { @so.parent_instance = nil }
-      end
-
-      it "has a working inherited virtual method #method_int8_in"
-      it "has a working inherited virtual method #method_int8_out"
-
-      it "has a working inherited virtual method #method_with_default_implementation" do
-        @so.method_with_default_implementation 104
-        assert_equal 104, @so.int
-      end
-
-      it "has a working inherited method #int8_in"
-      it "has a working inherited method #int8_out"
-
-      it "has a working inherited method #method" do
-        @so.set_property("int", 42)
-        @so.method
-        pass
-      end
-
-      it "has a working inherited method #method_array_in" do
-        @so.method_array_in [-1, 0, 1, 2]
-        pass
-      end
-
-      it "has a working inherited method #method_array_inout" do
-        res = @so.method_array_inout [-1, 0, 1, 2]
-        assert_equal [-2, -1, 0, 1, 2], res
-      end
-
-      it "has a working inherited method #method_array_out" do
-        res = @so.method_array_out
-        assert_equal [-1, 0, 1, 2], res
-      end
-
-      it "has a working inherited method #method_array_return" do
-        res = @so.method_array_return
-        assert_equal [-1, 0, 1, 2], res
-      end
-
-      it "has a working inherited method #none_in" do
-        @so.set_property("int", 42)
-        @so.none_in
-        pass
-      end
-
-      it "has a working inherited method #overridden_method" do
-        @so.set_property("int", 0)
-        @so.overridden_method
-        pass
-      end
+    it "does not have field accessors" do
+      assert_raises(NoMethodError) { instance.parent_instance }
+      assert_raises(NoMethodError) { instance.parent_instance = nil }
     end
   end
 
-  describe "Union" do
+  describe "GIMarshallingTests::SubSubObject" do
+    it "creates an instance using #new" do
+      so = GIMarshallingTests::SubSubObject.new 42
+      assert_instance_of GIMarshallingTests::SubSubObject, so
+    end
+
+    let(:instance) { GIMarshallingTests::SubSubObject.new 0 }
+
+    it "does not have field accessors" do
+      assert_raises(NoMethodError) { instance.parent_instance }
+      assert_raises(NoMethodError) { instance.parent_instance = nil }
+    end
+  end
+
+  describe "GIMarshallingTests::Union" do
     it "creates an instance with #new" do
       u = GIMarshallingTests::Union.new
       assert_instance_of GIMarshallingTests::Union, u
     end
 
-    describe "an instance" do
-      before do
-        @it = GIMarshallingTests::Union.new
-      end
+    let(:instance) { GIMarshallingTests::Union.new }
 
-      it "has a writable field long_" do
-        assert_equal 0, @it.long_
-        @it.long_ = 1056
-        assert_equal 1056, @it.long_
-      end
-
-      it "has a working method #inv" do
-        @it.long_ = 42
-        @it.inv
-        pass
-      end
-
-      it "has a working method #method" do
-        @it.long_ = 42
-        @it.method
-        pass
-      end
+    it "has a writable field long_" do
+      assert_equal 0, instance.long_
+      instance.long_ = 1056
+      assert_equal 1056, instance.long_
     end
+
+    it "has a working method #inv" do
+      instance.long_ = 42
+      instance.inv
+      pass
+    end
+
+    it "has a working method #method" do
+      instance.long_ = 42
+      instance.method
+      pass
+    end
+
+    it "has a working function #inout" do
+      skip "This function is defined in the header but not implemented"
+    end
+
+    it "has a working function #out" do
+      skip "This function is defined in the header but not implemented"
+    end
+
+    it "has a working function #returnv" do
+      u = GIMarshallingTests::Union.returnv
+      assert_instance_of GIMarshallingTests::Union, u
+    end
+  end
+
+  it "has a working function #array_enum_in" do
+    GIMarshallingTests.array_enum_in [:value1, :value2, :value3]
   end
 
   it "has a working function #array_fixed_inout" do
     res = GIMarshallingTests.array_fixed_inout [-1, 0, 1, 2]
-    assert_equal [2, 1, 0, -1], res
+    assert_equal [2, 1, 0, -1], res.to_a
   end
 
   it "has a working function #array_fixed_int_in" do
@@ -467,12 +619,12 @@ describe "GIMarshallingTests" do
 
   it "has a working function #array_fixed_int_return" do
     res = GIMarshallingTests.array_fixed_int_return
-    assert_equal [-1, 0, 1, 2], res
+    assert_equal [-1, 0, 1, 2], res.to_a
   end
 
   it "has a working function #array_fixed_out" do
     res = GIMarshallingTests.array_fixed_out
-    assert_equal [-1, 0, 1, 2], res
+    assert_equal [-1, 0, 1, 2], res.to_a
   end
 
   it "has a working function #array_fixed_out_struct" do
@@ -487,17 +639,21 @@ describe "GIMarshallingTests" do
 
   it "has a working function #array_fixed_short_return" do
     res = GIMarshallingTests.array_fixed_short_return
-    assert_equal [-1, 0, 1, 2], res
+    assert_equal [-1, 0, 1, 2], res.to_a
   end
 
-  it "has a working function #array_gvariant_(none)_in" do
+  it "has a working function #array_gvariant_container_in" do
+    skip
+  end
+  it "has a working function #array_gvariant_full_in" do
+    skip
+  end
+
+  it "has a working function #array_gvariant_none_in" do
+    skip "Test was disabled the return value was never correct"
     v1 = GLib::Variant.new_int32(27)
     v2 = GLib::Variant.new_string("Hello")
-    if GIMarshallingTests._builder.setup_method :array_gvariant_in
-      GIMarshallingTests.array_gvariant_in [v1, v2]
-    else
-      GIMarshallingTests.array_gvariant_none_in [v1, v2]
-    end
+    GIMarshallingTests.array_gvariant_none_in [v1, v2]
 
     pass
     # TODO: Can we determine that result should be an array?
@@ -510,19 +666,62 @@ describe "GIMarshallingTests" do
     pass
   end
 
+  it "has a working function #array_in_guint64_len" do
+    skip
+  end
+  it "has a working function #array_in_guint8_len" do
+    skip
+  end
+  it "has a working function #array_in_len_before" do
+    skip
+  end
+  it "has a working function #array_in_len_zero_terminated" do
+    skip
+  end
+  it "has a working function #array_in_nonzero_nonlen" do
+    skip
+  end
+
   it "has a working function #array_inout" do
     res = GIMarshallingTests.array_inout [-1, 0, 1, 2]
-    assert_equal [-2, -1, 0, 1, 2], res
+    assert_equal [-2, -1, 0, 1, 2], res.to_a
+  end
+
+  it "has a working function #array_inout_etc" do
+    skip
   end
 
   it "has a working function #array_out" do
     res = GIMarshallingTests.array_out
-    assert_equal  [-1, 0, 1, 2], res
+    assert_equal  [-1, 0, 1, 2], res.to_a
+  end
+
+  it "has a working function #array_out_etc" do
+    skip
   end
 
   it "has a working function #array_return" do
     res = GIMarshallingTests.array_return
-    assert_equal  [-1, 0, 1, 2], res
+    assert_equal  [-1, 0, 1, 2], res.to_a
+  end
+
+  it "has a working function #array_return_etc" do
+    skip
+  end
+  it "has a working function #array_simple_struct_in" do
+    skip
+  end
+
+  it "has a working function #array_string_in" do
+    GIMarshallingTests.array_string_in ["foo", "bar"]
+    pass
+  end
+
+  it "has a working function #array_struct_in" do
+    skip
+  end
+  it "has a working function #array_struct_take_in" do
+    skip
   end
 
   it "has a working function #array_uint8_in" do
@@ -536,6 +735,10 @@ describe "GIMarshallingTests" do
     pass
   end
 
+  it "has a working function #array_zero_terminated_inout" do
+    skip
+  end
+
   it "has a working function #array_zero_terminated_out" do
     res = GIMarshallingTests.array_zero_terminated_out
     assert_equal ["0", "1", "2"], res.to_a
@@ -546,6 +749,13 @@ describe "GIMarshallingTests" do
     assert_equal ["0", "1", "2"], res.to_a
   end
 
+  it "has a working function #array_zero_terminated_return_null" do
+    skip
+  end
+  it "has a working function #array_zero_terminated_return_struct" do
+    skip
+  end
+
   it "has a working function #boolean_in_false" do
     GIMarshallingTests.boolean_in_false false
     pass
@@ -608,8 +818,8 @@ describe "GIMarshallingTests" do
     ret = GIMarshallingTests.bytearray_full_return
     assert_instance_of GLib::ByteArray, ret
     assert_includes(
-	["0123".bytes.to_a, "\x001\xFF3".bytes.to_a],
-	ret.to_string.bytes.to_a)
+      ["0123".bytes.to_a, "\x001\xFF3".bytes.to_a],
+      ret.to_string.bytes.to_a)
   end
 
   it "has a working function #bytearray_none_in" do
@@ -620,6 +830,22 @@ describe "GIMarshallingTests" do
     pass
   end
 
+  it "has a working function #callback_multiple_out_parameters" do
+    skip
+  end
+  it "has a working function #callback_one_out_parameter" do
+    skip
+  end
+  it "has a working function #callback_return_value_and_multiple_out_parameters" do
+    skip
+  end
+  it "has a working function #callback_return_value_and_one_out_parameter" do
+    skip
+  end
+  it "has a working function #callback_return_value_only" do
+    skip
+  end
+
   it "has a working function #double_in" do
     GIMarshallingTests.double_in Float::MAX
     pass
@@ -726,6 +952,16 @@ describe "GIMarshallingTests" do
     assert_equal [-1, 0, 1, 2], arr.to_a
   end
 
+  it "has a working function #garray_uint64_none_in" do
+    skip
+  end
+  it "has a working function #garray_uint64_none_return" do
+    skip
+  end
+  it "has a working function #garray_utf8_container_inout" do
+    skip
+  end
+
   it "has a working function #garray_utf8_container_out" do
     res = GIMarshallingTests.garray_utf8_container_out
     assert_equal ["0", "1", "2"], res.to_a
@@ -826,6 +1062,16 @@ describe "GIMarshallingTests" do
     end
   end
 
+  it "has a working function #gerror_out" do
+    skip
+  end
+  it "has a working function #gerror_out_transfer_none" do
+    skip
+  end
+  it "has a working function #gerror_return" do
+    skip
+  end
+
   it "has a working function #ghashtable_int_none_in" do
     GIMarshallingTests.ghashtable_int_none_in(
       {-1 => 1, 0 => 0, 1 => -1, 2 => -2})
@@ -836,8 +1082,9 @@ describe "GIMarshallingTests" do
     assert_equal({-1 => 1, 0 => 0, 1 => -1, 2 => -2}, gh.to_hash)
   end
 
-  # This function is defined in the header but not implemented.
-  # it "has a working function #ghashtable_utf8_container_in"
+  it "has a working function #ghashtable_utf8_container_in" do
+    skip "This function is defined in the header but not implemented"
+  end
 
   it "has a working function #ghashtable_utf8_container_inout" do
     hsh = {"-1" => "1", "0" => "0", "1" => "-1", "2" => "-2"}
@@ -857,8 +1104,9 @@ describe "GIMarshallingTests" do
                  res.to_hash)
   end
 
-  # This function is defined in the header but not implemented.
-  # it "has a working function #ghashtable_utf8_full_in"
+  it "has a working function #ghashtable_utf8_full_in" do
+    skip "This function is defined in the header but not implemented"
+  end
 
   it "has a working function #ghashtable_utf8_full_inout" do
     hsh = {"-1" => "1", "0" => "0", "1" => "-1", "2" => "-2"}
@@ -912,8 +1160,12 @@ describe "GIMarshallingTests" do
     assert_equal [-1, 0, 1, 2], res.to_a
   end
 
-  # This function is defined in the header but not implemented.
-  # it "has a working function #glist_utf8_container_in"
+  it "has a working function #glist_uint32_none_in" do
+    skip
+  end
+  it "has a working function #glist_uint32_none_return" do
+    skip
+  end
 
   it "has a working function #glist_utf8_container_inout" do
     res = GIMarshallingTests.glist_utf8_container_inout ["0", "1", "2"]
@@ -930,9 +1182,6 @@ describe "GIMarshallingTests" do
     assert_equal ["0", "1", "2"], res.to_a
   end
 
-  # This function is defined in the header but not implemented.
-  # it "has a working function #glist_utf8_full_in"
-
   it "has a working function #glist_utf8_full_inout" do
     res = GIMarshallingTests.glist_utf8_full_inout ["0", "1", "2"]
     assert_equal ["-2", "-1", "0", "1"], res.to_a
@@ -967,6 +1216,37 @@ describe "GIMarshallingTests" do
     assert_equal ["0", "1", "2"], res.to_a
   end
 
+  it "has a working function #gptrarray_utf8_container_inout" do
+    skip
+  end
+  it "has a working function #gptrarray_utf8_container_out" do
+    skip
+  end
+  it "has a working function #gptrarray_utf8_container_return" do
+    skip
+  end
+  it "has a working function #gptrarray_utf8_full_inout" do
+    skip
+  end
+  it "has a working function #gptrarray_utf8_full_out" do
+    skip
+  end
+  it "has a working function #gptrarray_utf8_full_return" do
+    skip
+  end
+  it "has a working function #gptrarray_utf8_none_in" do
+    skip
+  end
+  it "has a working function #gptrarray_utf8_none_inout" do
+    skip
+  end
+  it "has a working function #gptrarray_utf8_none_out" do
+    skip
+  end
+  it "has a working function #gptrarray_utf8_none_return" do
+    skip
+  end
+
   it "has a working function #gslist_int_none_in" do
     GIMarshallingTests.gslist_int_none_in [-1, 0, 1, 2]
     pass
@@ -977,9 +1257,6 @@ describe "GIMarshallingTests" do
     assert_equal [-1, 0, 1, 2], res.to_a
   end
 
-  # This function is defined in the header but not implemented.
-  # it "has a working function #gslist_utf8_container_in"
-
   it "has a working function #gslist_utf8_container_inout" do
     res = GIMarshallingTests.gslist_utf8_container_inout ["0", "1", "2"]
     assert_equal ["-2", "-1", "0", "1"], res.to_a
@@ -995,9 +1272,6 @@ describe "GIMarshallingTests" do
     assert_equal ["0", "1", "2"], res.to_a
   end
 
-  # This function is defined in the header but not implemented.
-  # it "has a working function #gslist_utf8_full_in"
-
   it "has a working function #gslist_utf8_full_inout" do
     res = GIMarshallingTests.gslist_utf8_full_inout ["0", "1", "2"]
     assert_equal ["-2", "-1", "0", "1"], res.to_a
@@ -1053,6 +1327,10 @@ describe "GIMarshallingTests" do
     assert_equal ["0", "1", "2"], res.to_a
   end
 
+  it "has a working function #gtype_in" do
+    skip
+  end
+
   it "has a working function #gtype_inout" do
     none = GObject.type_from_name "void"
     res = GIMarshallingTests.gtype_inout none
@@ -1072,16 +1350,26 @@ describe "GIMarshallingTests" do
     assert_equal "void", name
   end
 
-  describe "#gvalue_in" do
-    it "works when passed a GObject::Value object" do
-      GIMarshallingTests.gvalue_in GObject::Value.wrap_ruby_value(42)
-      pass
-    end
+  it "has a working function #gtype_string_in" do
+    skip
+  end
+  it "has a working function #gtype_string_out" do
+    skip
+  end
+  it "has a working function #gtype_string_return" do
+    skip
+  end
+  it "has a working function #gvalue_flat_array" do
+    skip
+  end
+  it "has a working function #gvalue_flat_array_round_trip" do
+    skip
+  end
 
-    it "automatically converts a non-GValue object to a GValue" do
-      GIMarshallingTests.gvalue_in 42
-      pass
-    end
+  it "has a working function #gvalue_in" do
+    GIMarshallingTests.gvalue_in GObject::Value.wrap_ruby_value(42)
+    GIMarshallingTests.gvalue_in 42
+    pass
   end
 
   it "has a working function #gvalue_in_enum" do
@@ -1092,16 +1380,23 @@ describe "GIMarshallingTests" do
     pass
   end
 
-  describe "#gvalue_inout" do
-    it "works when passed a GObject::Value object" do
-      res = GIMarshallingTests.gvalue_inout GObject::Value.wrap_ruby_value(42)
-      assert_equal "42", res.ruby_value
-    end
+  it "has a working function #gvalue_in_with_type" do
+    skip
+  end
 
-    it "automatically converts a non-GValue input to a GValue" do
-      res = GIMarshallingTests.gvalue_inout 42
-      assert_equal "42", res.ruby_value
-    end
+  it "has a working function #gvalue_inout" do
+    res = GIMarshallingTests.gvalue_inout GObject::Value.wrap_ruby_value(42)
+    assert_equal "42", res.ruby_value
+
+    res = GIMarshallingTests.gvalue_inout 42
+    assert_equal "42", res.ruby_value
+  end
+
+  it "has a working function #gvalue_int64_in" do
+    skip
+  end
+  it "has a working function #gvalue_int64_out" do
+    skip
   end
 
   it "has a working function #gvalue_out" do
@@ -1109,11 +1404,19 @@ describe "GIMarshallingTests" do
     assert_equal 42, res.ruby_value
   end
 
+  it "has a working function #gvalue_out_caller_allocates" do
+    skip
+  end
+
   it "has a working function #gvalue_return" do
     res = GIMarshallingTests.gvalue_return
     assert_equal 42, res.ruby_value
   end
 
+  it "has a working function #init_function" do
+    skip
+  end
+
   it "has a working function #int16_in_max" do
     GIMarshallingTests.int16_in_max 0x7fff
     pass
@@ -1369,6 +1672,10 @@ describe "GIMarshallingTests" do
     assert_equal min_long, res
   end
 
+  it "has a working function #multi_array_key_value_in" do
+    skip
+  end
+
   it "has a working function #no_type_flags_in" do
     GIMarshallingTests.no_type_flags_in :value2
     pass
@@ -1394,6 +1701,16 @@ describe "GIMarshallingTests" do
     assert_equal :value2, res
   end
 
+  it "has a working function #overrides_struct_returnv" do
+    skip
+  end
+  it "has a working function #param_spec_out" do
+    skip
+  end
+  it "has a working function #param_spec_return" do
+    skip
+  end
+
   it "has a working function #pointer_in_return" do
     ptr = FFI::MemoryPointer.new 1
     res = GIMarshallingTests.pointer_in_return ptr
@@ -1412,6 +1729,10 @@ describe "GIMarshallingTests" do
     assert_equal 42, res.long_
   end
 
+  it "has a working function #return_gvalue_flat_array" do
+    skip
+  end
+
   it "has a working function #short_in_max" do
     GIMarshallingTests.short_in_max 0x7fff
     pass
@@ -1518,7 +1839,9 @@ describe "GIMarshallingTests" do
     assert_equal min_ssize_t, res
   end
 
-  it "has a working function #test_interface_test_int8_in"
+  it "has a working function #test_interface_test_int8_in" do
+    skip
+  end
 
   it "has a working function #time_t_in" do
     GIMarshallingTests.time_t_in 1234567890
@@ -1657,11 +1980,13 @@ describe "GIMarshallingTests" do
     assert_equal max_ulong, res
   end
 
-  # This function is defined in the header but not implemented.
-  # it "has a working function #union_inout"
+  it "has a working function #union_inout" do
+    skip "This function is defined in the header but not implemented"
+  end
 
-  # This function is defined in the header but not implemented.
-  # it "has a working function #union_out"
+  it "has a working function #union_out" do
+    skip "This function is defined in the header but not implemented"
+  end
 
   it "has a working function #union_returnv" do
     res = GIMarshallingTests.union_returnv
@@ -1694,8 +2019,10 @@ describe "GIMarshallingTests" do
     assert_nil res
   end
 
-  # This function is defined in the header but not implemented.
-  # it "has a working function #utf8_full_in"
+  it "has a working function #utf8_full_in" do
+    skip "This function is defined in the header but not implemented"
+  end
+
 
   it "has a working function #utf8_full_inout" do
     res = GIMarshallingTests.utf8_full_inout "const ♥ utf8"
@@ -1732,4 +2059,3 @@ describe "GIMarshallingTests" do
     assert_equal "const ♥ utf8", res
   end
 end
-
